package game

import (
	"fmt"
	"sirlab.de/go/knyt/syncdata"
	"sort"
)

func (gm *Game) populateSyncDataCb(syncData *syncdata.SyncData) {
	syncData.GameInfo = gm.populateGameInfo()
}

func (gm *Game) populateGameInfo() syncdata.GameInfo {
	gm.mu.Lock()
	defer gm.mu.Unlock()

	return syncdata.GameInfo{
		GameId:  gm.id,
		State:   gm.state,
		Phase:   gm.phase,
		Players: gm.getPlayers(),
		Round:   gm.getRoundInfo(),
	}
}

func (gm *Game) getPlayers() []syncdata.PlayerInfo {
	players := make([]syncdata.PlayerInfo, 0)
	for _, p := range gm.players {
		if p.isPlaying {
			players = append(players, syncdata.PlayerInfo{
				Id:     p.id,
				Name:   p.name,
				IsIdle: p.isIdle,
			})
		}
	}

	sort.Slice(players, func(a, b int) bool {
		return players[a].Name < players[b].Name
	})

	return players
}

func (gm *Game) getRoundInfo() *syncdata.RoundInfo {
	if gm.state != STATE_PLAY {
		return nil
	}

	quote := gm.quotes[gm.round.quoteId]
	if quote == nil {
		fmt.Printf("fatal error: quote id \"%s\" not found\n", gm.round.quoteId)
		return nil
	}

	sources := make([]syncdata.SourceInfo, 0)
	for _, source := range gm.round.sources {
		sources = append(sources, syncdata.SourceInfo{
			Id:   source.id,
			Name: source.name,
		})
	}

	roundInfo := syncdata.RoundInfo{
		Quote:      quote.GetQuote(),
		Sources:    sources,
		Selections: gm.getSelections(),
	}

	return &roundInfo
}

func (gm *Game) getSelections() map[string]bool {
	selections := make(map[string]bool, 0)
	for id, playerInfo := range gm.players {
		if !playerInfo.isPlaying {
			continue
		}

		if len(gm.round.selections[id]) > 0 {
			selections[id] = true
		} else if !playerInfo.isIdle {
			selections[id] = false
		}
	}

	return selections
}
